#include <errno.h>
#include <stdio.h>
#include "posix-queue.hxx"



PosixQueue::PosixQueue(const std::string& Name) : m_Name(std::string("/") + Name), m_mqd(-1), m_bVerbose(false)
{
}



PosixQueue::~PosixQueue()
{
  Close();
}



bool PosixQueue::Open()
{
  struct mq_attr attr;

  attr.mq_flags   = 0;      // O_NONBLOCK not set, i.e. blocking mode
  attr.mq_maxmsg  = 1024;   // See /proc/sys/fs/mqueue/msg_max.
  attr.mq_msgsize = 128;    // See /proc/sys/fs/mqueue/msgsize_max.
  attr.mq_curmsgs = 0;      // just for show - Queue is empty, of course.
  
  m_mqd = mq_open(m_Name.c_str(),O_RDWR | O_CREAT,0660,&attr);
  if (m_mqd == -1)
  {
    if (1)
    {
      perror("mq_open");
      fprintf(stderr,"\n  Check /proc/sys/fs/mqueue/msg_max and /proc/sys/fs/mqueue/msgsize_max.\n");
      fprintf(stderr,"  Also check the /dev/mqueue directory and remove any old queue if necessary.\n\n");
    }
    return true;
  }

  //
  // The Message Queue should now appear as "file" /dev/mqueue/Name.
  //

  if (m_bVerbose)
  {
    printf("message queue \"%s\" opened\n",m_Name.c_str());
    if (mq_getattr(m_mqd,&attr) == 0)
    {
      printf("\n  mq_flags   = %ld\n",attr.mq_flags);
      printf("  mq_maxmsg  = %ld\n",attr.mq_maxmsg);
      printf("  mq_msgsize = %ld\n",attr.mq_msgsize);
      printf("  mq_curmsgs = %ld\n\n",attr.mq_curmsgs);
    }
  }

  return false;
}



bool PosixQueue::Close()
{
  if (m_mqd != -1)
  {
    if (m_bVerbose)
    {
      printf("closing message queue \"%s\"...\n",m_Name.c_str());
    }
    mq_close(m_mqd);
    m_mqd = -1;
  }

  return false;
}



bool PosixQueue::Put(const char *pMessageBuffer, size_t MessageSize) const
{
  unsigned int uPrio = 0;

  if (pMessageBuffer == 0)
  {
    return true;
  }

  if (mq_send(m_mqd,pMessageBuffer,MessageSize,uPrio) == -1)
  {
    if (m_bVerbose)
    {
      perror("mq_send");
    }
    return true;
  }

  return false;
}



bool PosixQueue::Get(char *pMessageBuffer, size_t BufferSize, size_t& MessageSize) const
{
  if (pMessageBuffer == 0)
  {
    return true;
  }

  mqd_t k = mq_receive(m_mqd,pMessageBuffer,BufferSize,0);
  if (k == -1)
  {
    if (m_bVerbose)
    {
      perror("mq_receive");
    }
    return true;
  }
  MessageSize = k;

  return false;
}



long int PosixQueue::GetMessageCount() const
{
  struct mq_attr attr = { 0, 0, 0, 0 };

  if (mq_getattr(m_mqd,&attr) == 0)
  {
    return attr.mq_curmsgs;
  }

  return -1;
}



bool PosixQueue::SetBlockingMode() const
{ 
  struct mq_attr attr = { 0, 0, 0, 0 };
  
  return (mq_setattr(m_mqd,&attr,0) == -1);
}



bool PosixQueue::SetNonblockingMode() const
{
  struct mq_attr attr = { O_NONBLOCK, 0, 0, 0 };

  return (mq_setattr(m_mqd,&attr,0) == -1);
}



mqd_t PosixQueue::GetFileDescriptor() const
{
   return m_mqd;
}
